listo.class('ListoItems', function (listo, $) {
	const { ajax, hooks, template, data: config } = listo;

	const ANIM_SPEED = +config('animation_speed');

	return class {
		constructor(container) {
			this.collection = null;
			this.hideCompleted = null;

			this.$el = $(container);

			this.$el.on('click', '[data-listo=item]', e => this.onItemClick(e));
			this.$el.on('click', '[data-listo=item-toggle]', e => this.onItemToggleClick(e));
			this.$el.on('click', '[data-listo=item-check]', e => this.onItemCheckClick(e));
			this.$el.on('click', '[data-listo=item-collection]', e =>
				this.onItemCollectionClick(e)
			);
			this.$el.on('click', '[data-listo=description-toggle]', e =>
				this.onDescriptionToggleClick(e)
			);

			this.$el.on('click', '[data-listo=delete-item]', e => this.onDeleteItemClick(e));
			this.$el.on('click', '[data-listo=delete-item-cancel]', e =>
				this.onCancelDeleteItemClick(e)
			);
			this.$el.on('click', '[data-listo=delete-item-confirm]', e =>
				this.onConfirmDeleteItemClick(e)
			);
			this.$el.on('click', '[data-listo=edit-item]', e => this.onEditItemClick(e));
			this.$el.on('input', '[name=title]', e => this.onEditTitleInput(e));
			this.$el.on('input', '[name=description]', e => this.onEditDescriptionInput(e));
			this.$el.on('click', '[data-listo=cancel]', e => this.onCancelEditItemClick(e));
			this.$el.on('click', '[data-listo=save]', e => this.onSaveEditItemClick(e));

			// Initialize due date functions.
			new (listo.class('ListoItemDue'))(container);

			hooks.addAction('set_current_collection', data => {
				this.collection = data.id;
				this.update();
			});

			hooks.addAction('item_added', () => this.update());
			hooks.addAction('preferences_init', prefs => this.onPreferencesUpdated(prefs));
			hooks.addAction('preferences_updated', prefs => this.onPreferencesUpdated(prefs));
		}

		update() {
			this.$el.empty();

			// Show loading indicator if fetching takes more than 250ms.
			let loadingTimer = setTimeout(() => {
				let loadingTmpl = template(config('templates.item_list_loading'));
				this.$el.html(loadingTmpl());
			}, 250);

			ajax('listowp/v1/items', { collection: this.collection })
				.always(() => clearTimeout(loadingTimer)) // Hide loading indicator.
				.done(json => {
					this.render(json);
					hooks.doAction('items_loaded', json);
				});
		}

		render(items) {
			this.$el.empty();

			if (items instanceof Array && items.length) {
				let itemTmpl = template(config('templates.item'));
				items = hooks.applyFilters('item_map', items);
				items.forEach(item => this.$el.append(itemTmpl(item)));
			}
		}

		loadItem(id) {
			return ajax('listowp/v1/items', { id }).done(json => {});
		}

		updateItem(id, data = {}) {
			return ajax('listowp/v1/items', Object.assign(data, { id }), 'PATCH');
		}

		toggleItem(item, full) {
			let $item = $(item);
			let $items = $('[data-listo=item]');
			let fullClass = 'lo-task--full';

			if (true === full && !$item.hasClass(fullClass)) {
				$items.removeClass(fullClass);
				$item.addClass(fullClass);
			} else if (false === full && $item.hasClass(fullClass)) {
				$item.removeClass(fullClass);
			} else if ('undefined' === typeof full) {
				this.toggleItem(item, !$item.hasClass(fullClass));
			}
		}

		onPreferencesUpdated(prefs) {
			this.hideCompleted = +prefs.items_hide_done;
		}

		onItemClick(e) {
			e.preventDefault();
			e.stopPropagation();

			this.toggleItem(e.currentTarget, true);
		}

		onItemToggleClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let $item = $(e.currentTarget).closest('[data-listo=item]');
			this.toggleItem($item);
		}

		onItemCheckClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let $item = $(e.currentTarget).closest('[data-listo=item]');
			if ($item.data('saving')) return;
			$item.data('saving', 1);

			let id = $item.data('id');
			let rrule_id = $item.data('rrule_id');
			let status = +$item.data('status') ? 0 : 1;

			this.updateItem(id, { status, rrule_id }).done(json => {
				if (rrule_id && status) {
					this.update();
					$item.removeData('saving');
					hooks.doAction('item_updated', json[0]);
					hooks.doAction('maintenance');
				} else if ('done' === this.collection && !status) {
					$item.remove();
					$item.removeData('saving');
					hooks.doAction('item_updated', json[0]);
					hooks.doAction('maintenance');
				} else if (this.hideCompleted && status) {
					$item.remove();
					$item.removeData('saving');
					hooks.doAction('item_updated', json[0]);
					hooks.doAction('maintenance');
				} else {
					this.loadItem(id).done(json => {
						if (json instanceof Array && json.length) {
							let itemTmpl = template(config('templates.item'));
							let item = hooks.applyFilters('item_map', json[0]);

							$item.replaceWith(itemTmpl(item));
							hooks.doAction('item_updated', item);
						}
						$item.removeData('saving');
						hooks.doAction('maintenance');
					});
				}
			});
		}

		onItemCollectionClick(e) {
			e.preventDefault();
			e.stopPropagation();

			hooks.doAction('switch_collection', $(e.currentTarget).data('collection'));
		}

		onDeleteItemClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let $item = $(e.currentTarget).closest('[data-listo=item]');

			$item.find('[data-listo=delete-item]').hide();
			$item
				.find('[data-listo=delete-item-form]')
				.add($item.find('[data-listo=delete-item-cancel]'))
				.fadeIn(ANIM_SPEED);
		}

		onCancelDeleteItemClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let $item = $(e.currentTarget).closest('[data-listo=item]');

			$item
				.find('[data-listo=delete-item-form]')
				.add($item.find('[data-listo=delete-item-cancel]'))
				.fadeOut(ANIM_SPEED, () => {
					$item.find('[data-listo=delete-item]').show();
				});
		}

		onConfirmDeleteItemClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let $item = $(e.currentTarget).closest('[data-listo=item]');
			let id = $item.data('id');

			let $indicator = $item.find('[data-listo=delete-item-cancel] i');
			let $disabler = $('<div/>').css({
				position: 'absolute',
				top: 0,
				left: 0,
				right: 0,
				bottom: 0,
				background: 'rgba(255,255,255,0.5)'
			});

			$disabler.appendTo($item);
			$indicator.attr('class', 'fa-solid fa-circle-notch fa-spin');

			return ajax(`listowp/v1/items/${id}`, {}, 'DELETE')
				.always(() => {
					$disabler.remove();
					$indicator.attr('class', 'fa-solid fa-xmark');
				})
				.done(() => {
					if ($item.siblings().length) {
						$item.css({ opacity: 0 }).slideUp(ANIM_SPEED, () => $item.remove());
					} else {
						this.render([]);
					}

					hooks.doAction('maintenance');
				});
		}

		onDescriptionToggleClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let $item = $(e.currentTarget).closest('[data-listo=item]');
			let $desc = $item.find('[data-listo=description]').parent();
			let $descToggle = $item.find('[data-listo=description-toggle]');
			let $tooltipAltClass = 'lo-tip--alt';

			$desc.slideToggle();
			$descToggle.toggleClass($tooltipAltClass);
		}

		onEditItemClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let $item = $(e.currentTarget).closest('[data-listo=item]');
			let $itemCheck = $item.find('[data-listo=item-check]').parent();
			let $infoTitle = $item.find('[data-listo=title]').parent();
			let $infoDescription = $item.find('[data-listo=description]').parent();
			let $descriptionToggle = $item.find('[data-listo=description-toggle]');
			let $metaActions = $item.find('[data-listo=meta-actions]');
			let $editItem = $item.find('[data-listo=edit-item]');
			let $editForm = $item.find('[data-listo=edit-item-form]');
			let $editTitle = $editForm.find('[name=title]');
			let $editDescription = $editForm.find('[name=description]');
			let $editSave = $editForm.find('[data-listo=save]');
			let $editClass = 'lo-task--edit';

			$item.toggleClass($editClass);
			$metaActions.hide();
			$editItem.hide();
			$itemCheck.hide();
			$infoTitle.hide();
			$infoDescription.hide();
			$descriptionToggle.hide();
			$editForm.show();
			$editTitle.data('value', $editTitle.val());
			$editDescription.data('value', $editDescription.val()).listo_autosize();
			$editSave.attr('disabled', 'disabled');

			// Set focus on the title input.
			listo.focus($editTitle);
		}

		onEditTitleInput(e) {
			let $editForm = $(e.currentTarget).closest('[data-listo=edit-item-form]');
			let $editTitle = $editForm.find('[name=title]');
			let $editDescription = $editForm.find('[name=description]');
			let $editSave = $editForm.find('[data-listo=save]');

			if (!$editTitle.val().trim() || $editTitle.val() === $editTitle.data('value')) {
				if ($editDescription.val() === $editDescription.data('value')) {
					$editSave.attr('disabled', 'disabled');
					return;
				}
			}

			$editSave.removeAttr('disabled');
		}

		onEditDescriptionInput(e) {
			let $editForm = $(e.currentTarget).closest('[data-listo=edit-item-form]');
			let $editTitle = $editForm.find('[name=title]');
			let $editDescription = $editForm.find('[name=description]');
			let $editSave = $editForm.find('[data-listo=save]');

			if (!$editTitle.val().trim() || $editTitle.val() === $editTitle.data('value')) {
				if ($editDescription.val() === $editDescription.data('value')) {
					$editSave.attr('disabled', 'disabled');
					return;
				}
			}

			$editSave.removeAttr('disabled');
		}

		onCancelEditItemClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let $item = $(e.currentTarget).closest('[data-listo=item]');
			let $itemCheck = $item.find('[data-listo=item-check]').parent();
			let $infoTitle = $item.find('[data-listo=title]').parent();
			let $infoDescription = $item.find('[data-listo=description]').parent();
			let $descriptionToggle = $item.find('[data-listo=description-toggle]');
			let $metaActions = $item.find('[data-listo=meta-actions]');
			let $editItem = $item.find('[data-listo=edit-item]');
			let $editForm = $item.find('[data-listo=edit-item-form]');
			let $editTitle = $editForm.find('[name=title]');
			let $editDescription = $editForm.find('[name=description]');
			let $editSave = $editForm.find('[data-listo=save]');
			let $editClass = 'lo-task--edit';

			$item.toggleClass($editClass);
			$itemCheck.show();
			$metaActions.show();
			$editItem.show();
			$infoTitle.show();
			$infoDescription.hide();
			$descriptionToggle.show();
			$editForm.hide();
			$editTitle.val($editTitle.data('value'));
			$editDescription.val($editDescription.data('value'));
			$editSave.attr('disabled', 'disabled');
		}

		onSaveEditItemClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let $item = $(e.currentTarget).closest('[data-listo=item]');
			let $editForm = $item.find('[data-listo=edit-item-form]');
			let $editTitle = $editForm.find('[name=title]');
			let $editDescription = $editForm.find('[name=description]');
			let $editCancel = $editForm.find('[data-listo=cancel]');
			let $editSave = $editForm.find('[data-listo=save]');
			let id = $item.data('id');
			let title = $editTitle.val().trim();
			let description = $editDescription.val().trim();

			if ($editSave.data('saving')) return;
			$editSave.data('saving', 1);
			$editSave.add($editCancel).attr('disabled', 'disabled');

			return this.updateItem(id, { title, description })
				.always(() => {
					$editSave.removeData('saving');
					$editSave.add($editCancel).removeAttr('disabled');
				})
				.done(json => {
					let itemTmpl = template(config('templates.item'));
					let item = hooks.applyFilters('item_map', json[0]);

					$item.replaceWith(itemTmpl(item));

					hooks.doAction('item_updated', item);
					hooks.doAction('maintenance');
				});
		}
	};
});
