listo.class('ListoRoot', function (listo, $) {
	const { ajax, hooks, template, data: config } = listo;
	const { escapeHtml, nl2br } = listo.string;

	const ENABLE_COLLECTIONS = !!+config('enable_collections');
	const ANIM_SPEED = +config('animation_speed') || 0;

	return class extends listo.class('ListoAbstract') {
		constructor(container) {
			super(container);

			this.collectionsMap = {};
			this.currentCollection = config('last_viewed_collection');

			let rootTemplate = template(config('templates.root'));
			let rootTemplateData = { enable_collections: ENABLE_COLLECTIONS, lang: config('lang') };

			this.$el = $(container).html(rootTemplate(rootTemplateData));
			this.$wrapper = this.$el.find('[data-listo=wrapper]');
			this.$sidebar = this.$el.find('[data-listo=sidebar]');
			this.$sidebarCollapse = this.$el.find('[data-listo=sidebar-collapse]');
			this.$sidebarToggle = this.$el.find('[data-listo=sidebar-toggle]');
			this.$smartCollections = this.$el.find('[data-listo=smart-collections]');
			this.$collections = this.$el.find('[data-listo=collections]');
			this.$collectionHeader = this.$el.find('[data-listo=collection-header]');
			this.$items = this.$el.find('[data-listo=items]');
			this.$panels = this.$el.find('[data-listo=panel]');

			// Initialize collection header.
			new (listo.class('ListoCollectionHeader'))(this.$collectionHeader);

			// Initialize collection items.
			new (listo.class('ListoItems'))(this.$items);

			// Initialize new collection form.
			new (listo.class('ListoNewCollection'))({
				toggle: this.$el.find('[data-listo=new-collection]'),
				form: this.$el.find('[data-listo=new-collection-form]')
			});

			// Initialize new item form.
			new (listo.class('ListoNewItem'))({
				toggle: this.$el.find('[data-listo=new-item]'),
				form: this.$el.find('[data-listo=new-item-form]')
			});

			// Initialize sortable.
			this.sortable = new (listo.class('ListoSortable'))(this.$el);

			// Initialize preferences.
			new (listo.class('ListoPreferences'))(this.$el.find('[data-listo=preferences]'));

			hooks.addFilter('current_collection_id', () => this.currentCollection);
			hooks.addFilter('item_map', item => this.itemMap(item));

			hooks.addAction('switch_collection', id => this.loadItems(id));
			hooks.addAction('collection_added', data => this.onCollectionAdded(data));
			hooks.addAction('collection_updated', data => this.onCollectionUpdated(data));
			hooks.addAction('collection_deleted', id => this.onCollectionDeleted(id));
			hooks.addAction('items_loaded', () => this.onItemsLoaded());
			hooks.addAction('item_added', () => this.onItemAdded());
			hooks.addAction('item_moved', () => this.onItemMoved());
			hooks.addAction('preferences_updated', () => this.onPreferencesUpdated());
			hooks.addAction('gdpr_deleted', () => this.onGdprDeleted());
			hooks.addAction('count_items_updated', data => this.onCountItemsUpdated(data));
			hooks.addAction('maintenance', () => this.maintenance());

			$(document).on('keydown', e => {
				if ('KeyF' === e.code && e.ctrlKey && !e.altKey) {
					this.onWrapperToggleClick(e);
				}
			});

			this.maintenance().always(() => {
				if (ENABLE_COLLECTIONS) {
					this.loadCollections();
				} else {
					this.loadItems('all');
				}
			});
		}

		$collection(id) {
			return this.$smartCollections
				.add(this.$collections)
				.children(`[data-listo=collection][data-id=${id}]`);
		}

		$item(id) {
			return this.$items.children(`[data-listo=item][data-id=${id}]`);
		}

		loadCollections() {
			this.$panels.addClass('loading');

			return ajax('listowp/v1/collections')
				.done(json => {
					this.collectionsMap = {};

					this.$smartCollections.empty();
					this.$collections.empty();

					if (json instanceof Array) {
						// Sort collections based on the "order" key.
						json.sort((a, b) => +a.order - +b.order);

						let collectionTemplate = template(config('templates.collection'));

						json.forEach(collection => {
							this.collectionsMap[collection.id] = collection;

							let html = collectionTemplate(collection);

							if (collection.smart > 0) {
								this.$smartCollections.append(html);
							} else {
								this.$collections.append(html);
							}
						});

						this.loadItems(this.currentCollection);
					}

					hooks.doAction('collections_loaded');
				})
				.always(() => {
					this.$panels.removeClass('loading');
				});
		}

		itemMap(item) {
			if (!+this.currentCollection && this.currentCollection !== 'inbox') {
				let collection = this.collectionsMap[item.collection || 'inbox'];
				item.collection_id = collection.id;
				item.collection_title = collection.title;
				item.collection_icon = collection.icon;
				item.collection_initials = collection.initials;
			}

			return item;
		}

		loadItems(collection) {
			this.$panels.addClass('loading');
			this.currentCollection = collection;

			let $collections = this.$el.find('[data-listo=collection]');
			let $collection = this.$collection(collection);

			// Toggle active class for current collection.
			$collections.filter('.active').not($collection).removeClass('active');
			$collection.addClass('active');

			hooks.doAction('set_current_collection', this.collectionsMap[collection]);
		}

		loadItem(id) {
			return ajax('listowp/v1/items', { id }).done(json => {});
		}

		updateItem(id, data = {}) {
			return ajax('listowp/v1/items', Object.assign(data, { id }), 'PATCH').done(json => {});
		}

		setTimezone(value) {
			return ajax('listowp/v1/preferences', { id: 'timezone', value }, 'PATCH');
		}

		maintenance() {
			return ajax('listowp/v1/maintenance').done(json => {
				if (json instanceof Array && json[0] instanceof Object) {
					for (const id in json[0]) {
						let data = json[0][id];
						let $fields = this.$collection(id).find('[data-field^=count_items]');

						$fields.each((i, el) => {
							let $field = $(el);
							let field = $field.data('field');
							let count = data[`${field}_formatted`];

							$field.html(count);
							if ('count_items_due' === field) {
								+count ? $field.show() : $field.hide();
							}
						});

						hooks.doAction('count_items_updated', Object.assign(data, { id }));
					}
				}
			});
		}

		toggleSidebar(show) {
			let expandedClass = 'lo-sidebar--expanded';

			if ('undefined' !== typeof show && !+show) {
				$(document).off('click.listo-sidebar');
				this.$sidebar.removeClass(expandedClass);
				this.$sidebarToggle.find('span').html(this.$sidebarToggle.data('label-lists'));
			} else {
				this.$sidebar.addClass(expandedClass);
				this.$sidebarToggle.find('span').html(this.$sidebarToggle.data('label-tasks'));
				$(document).one('click.listo-sidebar', () => {
					this.$sidebar.removeClass(expandedClass);
					this.$sidebarToggle.find('span').html(this.$sidebarToggle.data('label-lists'));
				});
			}
		}

		onWrapperToggleClick(e) {
			e.preventDefault();
			e.stopPropagation();

			this.$wrapper.toggleClass('lo-wrapper--fullscreen');
		}

		onSidebarCollapseClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let className = 'lo-sidebar--collapsed';
			if (this.$sidebar.hasClass(className)) {
				this.$sidebar.removeClass(className);
				this.$sidebarCollapse.removeClass('lo-tip--alt');
			} else {
				this.$sidebar.addClass(className);
				this.$sidebarCollapse.addClass('lo-tip--alt');
			}
		}

		onSidebarToggleClick(e) {
			e.preventDefault();
			e.stopPropagation();

			this.toggleSidebar(!this.$sidebar.hasClass('lo-sidebar--expanded'));
		}

		onCollectionAdded(data = {}) {
			if ('object' === typeof data && data.id) {
				this.currentCollection = data.id;
				this.loadCollections();
			}
		}

		onCollectionUpdated(data = {}) {
			if ('object' === typeof data && data.id) {
				let $collection = this.$collection(data.id);
				let collection = this.collectionsMap[data.id];

				// Update collection information.
				for (const key in data) {
					let value = data[key];

					// Update local cache.
					collection[key] = value;

					// Update icon if necessary.
					if ('icon' === key) {
						let $icon = $collection.find('[data-field=icon]');
						let classList = $icon.get(0).classList;
						let iconClasses = [...classList].filter(i => i.match(/^fa-/));
						classList.remove(...iconClasses);
						if (value) classList.add(...value.split(' '));
						continue;
					}

					// Escape HTML tags and replace newlines with '<br>' tags.
					if (['initials', 'title', 'description'].indexOf(key) > -1) {
						value = nl2br(escapeHtml(value));
					}

					$collection.find(`[data-field=${key}]`).html(value);
				}
			}
		}

		onCollectionDeleted(id) {
			this.$collection(id).remove();
			this.currentCollection = 'inbox';
			this.loadCollections();

			hooks.doAction('maintenance');
		}

		onCollectionClick(e) {
			e.preventDefault();
			e.stopPropagation();

			let $collection = $(e.currentTarget);

			this.loadItems($collection.data('id'));
			this.toggleSidebar(false);
		}

		onItemsLoaded() {
			this.sortable.init(this.currentCollection);
		}

		onItemAdded() {
			hooks.doAction('maintenance');
		}

		onItemMoved() {
			hooks.doAction('maintenance');
		}

		onPreferencesUpdated() {
			ENABLE_COLLECTIONS ? this.loadCollections() : this.loadItems('all');
		}

		onGdprDeleted() {
			location.reload(true);
		}

		onCountItemsUpdated(data) {
			if (this.collectionsMap[data.id]) {
				Object.assign(this.collectionsMap[data.id], data);
			}
		}

		onPreferencesToggleClick(e) {
			hooks.doAction('preferences_toggle');
		}

		onDropdownToggleClick(e) {
			let $toggle = $(e.currentTarget);
			let $menu = $toggle.siblings('[data-listo=dropdown-menu]');

			if ($menu.is(':hidden')) {
				e.stopPropagation();
				$menu.show();
				$(document).one('click', () => $menu.hide());
			} else {
				$menu.hide();
			}
		}

		onDropdownMenuClick(e) {
			e.stopPropagation();
		}

		onPreferencesClick(e) {
			e.stopPropagation();
		}
	};
});
