(function (listo) {
	const string = {
		/**
		 * Capitalize a string.
		 *
		 * @param {string} str
		 * @returns {string}
		 */
		capitalize(str = '') {
			return str.replace(/(?:^|-)([a-z0-9])/gi, function (match, letter) {
				return letter.toUpperCase();
			});
		},

		/**
		 * Pad a string to a certain length.
		 *
		 * @param {string} str
		 * @param {number} length
		 * @param {string} char
		 * @returns {string}
		 */
		pad(str = '', length = 0, char = '') {
			if (char.length) {
				str = str + '';
				while (str.length < length) {
					str = char + str;
				}
			}

			return str;
		},

		/**
		 * Escape HTML tags.
		 *
		 * @param {string} html
		 * @returns {string}
		 */
		escapeHtml(html) {
			return html.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;');
		},

		/**
		 * Replace newlines with <br> tags.
		 *
		 * @param {string} text
		 * @returns {string}
		 */
		nl2br(text) {
			return text.replace(/\r?\n/g, '<br>');
		},

		/**
		 * Convert 12-hours format into 24-hours format.
		 *
		 * @param {string|number} hours
		 * @param {string} ampm
		 * @returns {string}
		 */
		h12to24(hours, ampm) {
			hours = +hours === 12 ? (ampm === 'pm' ? 12 : 0) : ampm === 'pm' ? +hours + 12 : hours;

			return string.pad(String(hours), 2, '0');
		},

		/**
		 * Convert 24-hours format into 12-hours format.
		 *
		 * @param {string|number} hours
		 * @returns {string}
		 */
		h24to12(hours) {
			hours = +hours > 12 ? +hours - 12 : +hours > 0 ? hours : 12;

			return string.pad(String(hours), 2, '0');
		},

		/**
		 * Check am or pm on a 24-hours format.
		 *
		 * @param {string|number} hours
		 * @returns {string}
		 */
		h24ampm(hours) {
			return +hours > 11 ? 'pm' : 'am';
		},

		/**
		 * Parse date string.
		 *
		 * @param {string} dateStr
		 * @returns {Object}
		 */
		parseDate(dateStr) {
			if (typeof dateStr !== 'string') {
				return dateStr;
			}

			const reDate = /^(\d{4})-(\d{2})-(\d{2}) (\d{2}):(\d{2}):(\d{2})$/;
			const parsedDate = dateStr.match(reDate);
			if (!parsedDate) {
				return dateStr;
			}

			return {
				date: `${parsedDate[1]}-${parsedDate[2]}-${parsedDate[3]}`,
				time: `${parsedDate[4]}:${parsedDate[5]}:${parsedDate[6]}`,
				hours: parsedDate[4],
				hours12h: +parsedDate[4] % 12 ? string.pad(+parsedDate[4] % 12, 2, '0') : '12',
				minutes: parsedDate[5],
				seconds: parsedDate[6],
				ampm: +parsedDate[4] < 12 ? 'am' : 'pm'
			};
		}
	};

	Object.assign(listo, { string });
})(window.listo);
